package com.softslate.commerce.businessobjects.core;

import java.util.Map;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.LuceneIndexer;

/**
 * A class which may be run from the command line to reindex the Lucene index.
 * This command is useful in stores where product data changes regularly, as a
 * way to synchronize the database with the Lucene index. See the accompanying
 * <code>commandLineLuceneReindex.sh</code> file for a Unix shell script that
 * may be used to fire off the example.
 * 
 * This example uses the <code>CommandLineSupport</code> class to initialize
 * all of the system objects such as the <code>BusinessObjectFactory</code>
 * and the <code>DAOFactory</code>. It then initializes
 * <code>LuceneProcessor</code>, calling the <code>reindex()</code> method
 * to execute the reindexing.
 * 
 * @author dtobey
 * 
 */
public class CommandLineReindexLucene extends CommandLineSupport {

	static Log log = LogFactory.getLog(CommandLineReindexLucene.class);

	/**
	 * Main method, which gets invoked when executing this class from the
	 * command line.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		// Log the start of the process
		if (log.isInfoEnabled())
			log.info("Starting Lucene reindexing.");

		// This command accepts 0 or 1 arguments. The argument may represent the
		// path on the server to the Lucene index directory. However, if the
		// luceneIndexDir setting in sscSetting does not begin with "/WEB-INF",
		// the value of the database setting will be used instead.
		if (args == null || args.length > 1) {
			if (log.isErrorEnabled())
				log
						.error("Wrong number of arguments. 0 or 1 arguments needed.");
			System.out
					.println("Usage: ./commandLineReindexLucene.sh [<path_to_Lucene_index_directory>]");
		}

		// Initilize the system objects
		Map systemObjects = initializeSystem("Lucene Reindex Admin User");
		if (systemObjects == null) {
			if (log.isErrorEnabled())
				log.error("Initialization failed ... exiting.");
			return;
		}

		// Unpack the system objects needed by this process
		BusinessObjectFactory bof = (BusinessObjectFactory) systemObjects
				.get("bof");
		// Properties appSettings = (Properties)
		// systemObjects.get("appSettings");
		// Properties appComponents = (Properties) systemObjects
		// .get("appComponents");
		// DAOFactory daoFactory = (DAOFactory) systemObjects.get("daoFactory");
		// Settings settings = (Settings) systemObjects.get("settings");
		// User user = (User) systemObjects.get("user");

		try {

			// Set the index directory
			Map parameters = new HashMap();
			if (args != null && args.length == 1) {
				if (log.isInfoEnabled())
					log.info("Index directory provided on command line: " + args[0]);
				parameters.put("indexDir", args[0]);
			}
			
			// Call the processor to do the reindexing
			LuceneIndexer processor =  bof
					.createObject(LuceneIndexer.class);
			Map results = processor.reindex(parameters);

			// Output the results
			if (log.isInfoEnabled())
				log.info("Lucene reindexing results: " + results);

		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Reindexing Lucene failed ... exiting.", e);
			return;
		}

		// Log the end of process
		if (log.isInfoEnabled())
			log.info("Ending Lucene reindexing.");
	}
}
