package com.softslate.commerce.businessobjects.events;

import java.util.Collection;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.eventbus.Subscribe;
import com.softslate.commerce.businessobjects.core.ApplicationMessage;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.Payment;
import com.softslate.commerce.businessobjects.payment.PaymentProcessor;

public class CapturePaymentTriggerImpl extends BaseTrigger implements CapturePaymentTrigger {

	static Log log = LogFactory.getLog(CapturePaymentTriggerImpl.class);

	protected Order order;
	protected Map incomingParameters;
	protected Map results;
	protected Settings settings;
	protected BusinessObjectFactory bof;
	protected Boolean preventStatusChangeIfPaymentCaptureFails;

	public boolean getSwallowExceptions() {
		return false;
	}

	@Subscribe
	public void processCapturePayment(ChangingOrderEvent coe) throws Exception {
		log.info("Trigger for capturing payment.");
		initializeFromEvent(coe);
		Order order = coe.getOrder();
		Map incomingParameters = coe.getIncomingParameters();
		Settings settings = coe.getBusinessProcessor().getSettings();
		String orderStatusForCapturingPayments = settings.getValue("orderStatusForCapturingPayments");
		String capturePaymentsWhenOrderStatusChanges = settings.getValue("capturePaymentsWhenOrderStatusChanges");

		if (orderStatusForCapturingPayments != null && order != null && order.getStatus() != null
				&& !order.getStatus().equals(orderStatusForCapturingPayments) && incomingParameters != null
				&& incomingParameters.get("status") != null
				&& incomingParameters.get("status").equals(orderStatusForCapturingPayments)
				&& capturePaymentsWhenOrderStatusChanges.equals("1")) {
			log.info("Processing payment capture for orderID: " + order.getOrderID());

			// Calls processTrigger below, after BaseTrigger optionally creates
			// a new thread or sets up exceptions to be swallowed.
			process();
		}
	}

	public void initializeFromEvent(ChangingOrderEvent coe) throws Exception {
		order = coe.getOrder();
		incomingParameters = coe.getIncomingParameters();
		results = coe.getResults();
		settings = coe.getBusinessProcessor().getSettings();
		bof = coe.getBusinessProcessor().getBusinessObjectFactory();
		preventStatusChangeIfPaymentCaptureFails = ("1").equals(settings
				.getValue("preventStatusChangeIfPaymentCaptureFails"));
	}

	public void processTrigger() throws Exception {
		String paymentProcessorClassName = findPaymentProcessorClassName(order);
		if (paymentProcessorClassName != null) {
			try {
				PaymentProcessor processor = (PaymentProcessor) bof
						.createObjectFromClassName(paymentProcessorClassName);
				Map paymentResults = processor.processPaymentCapture(order);
				if (paymentResults == null) {
				} else if ("0".equals(paymentResults.get("resultCode"))) {
					addMessage(paymentResults, ApplicationMessage.Level.SUCCESS,
							(String) paymentResults.get("resultMessage"));
				} else if ("-1".equals(paymentResults.get("resultCode"))
						|| "-3".equals(paymentResults.get("resultCode"))) {
					addMessage(paymentResults, ApplicationMessage.Level.WARNING,
							(String) paymentResults.get("resultMessage"));
				} else if (!"0".equals(paymentResults.get("resultCode"))) {
					addMessage(paymentResults, ApplicationMessage.Level.ERROR,
							(String) paymentResults.get("resultMessage"));
					revertStatusIfCalledForBySetting();
				}
				if (paymentResults != null) {
					results.putAll(paymentResults);
				}
			} catch (Exception e) {
				String error = "Payment not captured for order #" + order.getOrderNumber() + ": "
						+ paymentProcessorClassName + ": " + e.toString();
				log.error(error, e);
				addMessage(results, ApplicationMessage.Level.WARNING, error);
			}
		}
	}

	protected String findPaymentProcessorClassName(Order order) {
		String paymentProcessorClassName = null;
		for (Payment payment : (Collection<Payment>) order.getPayments()) {
			if (payment.getProcessorClassName() != null && payment.getProcessorClassName().length() > 0) {
				paymentProcessorClassName = payment.getProcessorClassName();
				break;
			}
		}
		return paymentProcessorClassName;
	}

	protected void revertStatusIfCalledForBySetting() {
		if (preventStatusChangeIfPaymentCaptureFails != null && preventStatusChangeIfPaymentCaptureFails) {
			incomingParameters.put("status", order.getStatus());
			String message = "Order status reverted to: " + order.getStatus();
			log.info(message);
			addMessage(results, ApplicationMessage.Level.WARNING, message);
		}
	}

}
