package com.softslate.commerce.businessobjects.shipping;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderItemAttributeBean;
import com.softslate.commerce.businessobjects.order.OrderShippingRule;
import com.softslate.commerce.businessobjects.product.Manufacturer;
import com.softslate.commerce.businessobjects.product.ManufacturerProcessor;
import com.softslate.commerce.daos.order.OrderGatewayDAO;

/**
 * Abstract class that provides methods useful for
 * <code>ShippingProcessor</code> implementations.
 * 
 * @author David Tobey
 * 
 */
public abstract class BaseShippingProcessor extends BaseBusinessProcessor
		implements ShippingProcessor {

	static Log log = LogFactory.getLog(BaseShippingProcessor.class);

	private String shippingOption = null;

	public String getShippingOption() {
		return shippingOption;
	}

	public void setShippingOption(String string) {
		shippingOption = string;
	}

	private Collection availableOptions = null;

	public Collection getAvailableOptions() {
		return availableOptions;
	}

	public void setAvailableOptions(Collection availableOptions) {
		this.availableOptions = availableOptions;
	}

	/*
	 * A Collection of all the drop shippers with items in the cart. Any items
	 * not associated with a drop shipper will be grouped together and a query
	 * for their rates will be sent to the carrier.
	 */
	public Collection dropShipperIDs = new LinkedHashSet();

	public Collection dropShippers = new LinkedHashSet();

	/*
	 * The current drop shipper being processed. A drop shipper is a
	 * manufacturer whose "isDropShipper" flag is set to yes.
	 */
	public Integer currentDropShipperID = null;

	public Collection loadShippingOptions(Map argument) throws Exception {

		// Call the implementation to load the options
		Collection options = null;
		try {
			options = loadShippingOptions();
		} catch (Exception e) {
			BasicShippingProcessor bsp = (BasicShippingProcessor) getBusinessObjectFactory().createObjectFromClassName(
					"com.softslate.commerce.businessobjects.shipping.BasicShippingProcessor");
			if (bsp.contingencyShippingMethodCodeDefined()) {
				log.error("Exception loading shipping options: " + e);
				options = bsp.loadContingencyShippingOption();
			} else {
				throw e;
			}
		}
		if (options == null) {
			return options;
		}

		// Derive the option's processorClass and processorName props
		String processorClass = this.getClass().getName();
		int lastdot = processorClass.lastIndexOf(".");
		if (lastdot != -1) {
			processorClass = processorClass.substring(lastdot + 1);
		}
		String processorName = processorClass;
		processorName = processorName.replaceFirst("ShippingProcessor", "");

		Iterator i = options.iterator();
		while (i.hasNext()) {
			Map option = (Map) i.next();
			
			// Assign the processorClass property if it is null, or if it's the contingency method. The
			// contingency method by default will have BasicShippingProcessor as the processorClass but we want it to
			// reflect the processor that failed instead, so that we don't have to activate BasicShippingProcessor.
			if (option.get("processorClass") == null
					|| (option.get("code") != null && option.get("code").equals(
							getSettings().getValue("contingencyShippingMethodCode")))) {
				option.put("processorClass", processorClass);
			}
			if (option.get("processorName") == null) {
				option.put("processorName", processorName);
			}
		}

		// Sort the options from lowest price to highest
		try {
			List optionsList = (List) options;
			Collections.sort(optionsList, new ShippingOptionComparator());
			return optionsList;
		} catch (Exception e) {
			if (log.isWarnEnabled())
				log.warn("Error sorting shipping options: ", e);
		}

		return options;
	}

	public abstract Collection loadShippingOptions() throws Exception;

	public abstract int processShipping() throws Exception;

	/**
	 * Assists with processing of shipping charges. Copies the incoming
	 * <code>Map</code> of parameters to this
	 * <code>BaseShippingProcessor</code> instance and calls
	 * <code>processShipping()</code> followed by
	 * <code>updateTotalsAndStatuses()</code> to update the order's totals.
	 * Finally, writes the new totals to the database using
	 * {@link com.softslate.commerce.daos.order.OrderGatewayDAO#updateOrderAndDelivery}.
	 */
	public Map processShipping(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing shipping.");
		Map results = new HashMap();
		int result = 0;
		try {
			// Copy the incoming parameters to this object.
			BeanUtils.copyProperties(this, parameters);

			if (!verifyOption()) {
				results.put("resultMessage", "Shipping rate not found.");
				results.put("resultCode", "1");
				return results;
			}

			if (log.isDebugEnabled())
				log.debug("Update the order with the option selected.");
			updateOrderWithOption();

			if (log.isDebugEnabled())
				log.debug("Update the order with the option's price.");
			updateOrderWithOptionPrice();

			if (log.isDebugEnabled())
				log.debug("Implementation-specific processing.");
			result = processShipping();

			if (log.isDebugEnabled())
				log.debug("Updating totals and statuses in memory.");
			updateTotalsAndStatuses();

			if (log.isDebugEnabled())
				log.debug("Updating totals and statuses in database.");
			updateTotalsAndStatusesInDatabase();

		} catch (Exception e) {
			if (getUser().getOrder().getCompleted() == null) {
				getUser().getOrder().setStatus("Corrupted");
			}
			throw e;
		}
		results.put("resultCode", Integer.toString(result));
		return results;
	}

	/**
	 * Verifies that the option the user selected is allowed by checking it
	 * against the Collection of avialable options.
	 * 
	 * @return true, if the option the user selected is allowed.
	 */
	public boolean verifyOption() {

		// Verify that the option selected is one of the available ones
		if (getAvailableOptions() == null) {
			return false;
		}

		Iterator i = getAvailableOptions().iterator();
		while (i.hasNext()) {
			Map rate = (Map) i.next();
			String service = (String) rate.get("code");
			if (service != null && service.equals(getShippingOption())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Sets the option the user selected in the user's order, by default in the
	 * order delivery's status details field.
	 * 
	 */
	public void updateOrderWithOption() {

		// Set the option that was selected in the user's order
		if (log.isDebugEnabled())
			log.debug("Setting the option in the order.");
		getUser().getOrder().setShippingMethod(getShippingOption());

		// For legacy installations, set it in the delivery.statusDetails field.
		getUser().getOrder().firstDelivery().setStatusDetails(
				getShippingOption());

		// Replace with the human-readable name.
		if (getSettings().getValue("storeShippingMethodNameWithOrder") != null
				&& getSettings().getValue("storeShippingMethodNameWithOrder")
						.equals("1") && getAvailableOptions() != null) {
			Iterator i = getAvailableOptions().iterator();
			while (i.hasNext()) {
				Map rate = (Map) i.next();
				String service = (String) rate.get("code");
				String name = (String) rate.get("name");
				if (service != null && service.equals(getShippingOption())
						&& name != null) {
					// Reset the option with the human-readable name.
					if (log.isDebugEnabled())
						log.debug("Resetting the option in the order.");
					getUser().getOrder().setShippingMethod(name);
					getUser().getOrder().firstDelivery().setStatusDetails(name);
				}
			}
		}
	}

	/**
	 * Sets the shipping price in the order based on the option the user
	 * selected.
	 * 
	 */
	public void updateOrderWithOptionPrice() {

		// Find the option and use its price
		if (getAvailableOptions() == null) {
			return;
		}

		Double priceDouble = null;
		Iterator i = getAvailableOptions().iterator();
		while (i.hasNext()) {
			Map rate = (Map) i.next();
			String service = (String) rate.get("code");
			if (service != null && service.equals(getShippingOption())) {
				priceDouble = (Double) rate.get("priceDouble");
				// Support for Foraker's FedEx processor
				if (priceDouble == null) {
					String rawPrice = (String) rate.get("rawPrice");
					if (rawPrice != null) {
						priceDouble = (Double) new Double(rawPrice);
					}
				}
				break;
			}
		}
		Double priceDoubleRnd = null;
		if (priceDouble != null) {
			priceDoubleRnd = utils().roundForMoney(priceDouble);
		}
		getUser().getOrder().firstDelivery().setShipping(priceDoubleRnd);

	}

	/**
	 * Updates the totals and statuses of a user's order with a new shipping
	 * charge included. The user's <code>Order</code> and that order's first
	 * <code>OrderDelivery</code> are updated with new price totals, and
	 * subtotals.
	 * 
	 * @throws Exception
	 */
	public void updateTotalsAndStatuses() throws Exception {
		// Initialize totals properties
		Double dShip = getUser().getOrder().firstDelivery().getShipping();
		Double dTax = getUser().getOrder().firstDelivery().getTax();
		Double dSub = getUser().getOrder().firstDelivery().getSubtotal();
		if (dShip == null) {
			dShip = new Double(0);
		}
		if (dTax == null) {
			dTax = new Double(0);
		}
		if (dSub == null) {
			dSub = new Double(0);
		}
		// Set the order's tax and shipping based on the delivery's tax and
		// shipping
		dShip = utils().roundForMoney(dShip);
		if (log.isDebugEnabled())
			log.debug("Setting shipping total: " + dShip);
		getUser().getOrder().setShipping(dShip);
		getUser().getOrder().setTax(dTax);
		getUser().getOrder().setSubtotal(dSub);
		// Set order total and order delivery total
		getUser().getOrder().firstDelivery().setTotal(
				new Double(dSub.doubleValue() + dTax.doubleValue()
						+ dShip.doubleValue()));
		getUser().getOrder().setTotal(
				new Double(getUser().getOrder().getSubtotal().doubleValue()
						+ getUser().getOrder().getTax().doubleValue()
						+ getUser().getOrder().getShipping().doubleValue()));
		if (getUser().getOrder().getCompleted() == null
				|| getUser().getOrder().getCompleted().length() == 0) {
			getUser().getOrder().firstDelivery().setStatus("Pending");
			getUser().getOrder().setStatus("Shipping processed");
		}
		getUser().getOrder().firstDelivery().addFormatting(
				getUser().getLocale(),
				(String) getSettings().getValue("currencyCode"));
		getUser().getOrder().addFormatting(getUser().getLocale(),
				(String) getSettings().getValue("currencyCode"));
		return;
	}

	public void updateTotalsAndStatusesInDatabase() throws Exception {
		// Update the order and the delivery in the database
		OrderGatewayDAO orderGatewayDAO =  getDaoFactory()
				.createDAO(OrderGatewayDAO.class);
		orderGatewayDAO.updateOrderAndDelivery(true, getUser().getOrder(),
				getUser().getOrder().firstDelivery());

	}

	/**
	 * Returns a Collection of Manufacturer objects associated with the order
	 * items in the given order, whose "isDropShipper" flag is set to true. Used
	 * by shipping processors such as the UPS processor to query for rates
	 * separately for each drop shipper.
	 * 
	 * @param order
	 * @return A Collection of Manufacturer objects associated with the order
	 *         items in the given order, whose "isDropShipper" flag is set to
	 *         true
	 * @throws Exception
	 */
	public Collection retrieveDropShippersFromOrder(Order order)
			throws Exception {

		if (order == null || order.firstDelivery() == null
				|| order.firstDelivery().getOrderItemCollection() == null) {
			return dropShippers;
		}

		Iterator i = order.firstDelivery().getOrderItemCollection().iterator();
		while (i.hasNext()) {
			OrderItem oi = (OrderItem) i.next();

			if (oi.getManufacturerID() == null
					|| oi.getManufacturerID().intValue() == 0) {
				continue;
			}

			// Load the manufacturer record (note, only the id is saved with the
			// order)
			ManufacturerProcessor processor =  getBusinessObjectFactory()
					.createObject(ManufacturerProcessor.class);
			HashMap map = new HashMap();
			map.put("manufacturerID", oi.getManufacturerID());
			Manufacturer manufacturer = processor.loadManufacturerFromID(map);

			if (manufacturer != null && manufacturer.getIsDropShipper()
					&& manufacturer.getIsActive()
					&& manufacturer.getCity() != null
					&& manufacturer.getState() != null
					&& manufacturer.getPostalCode() != null
					&& manufacturer.getCountry() != null
					&& manufacturer.getCity().length() > 0
					&& manufacturer.getState().length() > 0
					&& manufacturer.getPostalCode().length() > 0
					&& manufacturer.getCountry().length() > 0) {

				// Make sure the manufacturer is assigned to the item for later
				// processes
				oi.setManufacturer(manufacturer);

				// Add the supplier to the collection
				if (!dropShipperIDs.contains(new Integer(manufacturer
						.getManufacturerID()))) {
					dropShipperIDs.add(new Integer(manufacturer
							.getManufacturerID()));
					dropShippers.add(manufacturer);
				}

				if (log.isDebugEnabled())
					log.debug("Added manufacturer to dropShippers in order: "
							+ manufacturer.getCode());
			}
		}

		return dropShippers;

	}

	public boolean isPackageOkToCreate(OrderShippingRule osr) {
		boolean packageMatchesDropShipper = osr.getRuleType().equals(
				"[package]")
				&& !osr.getCode().equals("[generic]")
				&& currentDropShipperID != null
				&& osr.getOrderItem() != null
				&& osr.getOrderItem().getManufacturer() != null
				&& osr.getOrderItem().getManufacturerID().intValue() == currentDropShipperID
						.intValue();
		boolean packageNotDropShipped = osr.getRuleType().equals("[package]")
				&& currentDropShipperID == null
				&& !osr.getCode().equals("[generic]")
				&& (osr.getOrderItem() == null
						|| osr.getOrderItem().getManufacturer() == null || !dropShipperIDs
						.contains(osr.getOrderItem().getManufacturerID()));
		boolean doingGenericPackage = osr.getRuleType().equals("[package]")
				&& osr.getCode().equals("[generic]");
		boolean createThePackage = packageMatchesDropShipper
				|| packageNotDropShipped || doingGenericPackage;
		if (log.isDebugEnabled())
			log.debug("Package booleans: packageMatchesDropShipper: "
					+ packageMatchesDropShipper + " packageNotDropShipped: "
					+ packageNotDropShipped + " doingGenericPackage: "
					+ doingGenericPackage + " createThePackage: "
					+ createThePackage);
		return createThePackage;
	}

	public Double findPackageWeight(OrderShippingRule osr) {

		boolean foundAnItem = false;

		// For the generic package, we have to add the weights of those items
		// that match the current drop shipper but are not in some other
		// package.
		double weight = osr.getPkgWeight().doubleValue();
		if (osr.getCode().equals("[generic]") && dropShippers != null
				&& !dropShippers.isEmpty()) {
			weight = 0;

			// First we need a list of the quantities of each item that is
			// accounted for with another package - we don't want to take these
			// into account since we're doing the generic package now.
			Map itemQuantitiesInOtherPackages = new HashMap();
			if (getUser().getOrder().getOrderShippingRules() != null
					&& !getUser().getOrder().getOrderShippingRules().isEmpty()) {
				Iterator i = getUser().getOrder().getOrderShippingRules()
						.iterator();
				while (i.hasNext()) {
					OrderShippingRule osr2 = (OrderShippingRule) i.next();
					boolean itemInAnotherPackage = osr2.getRuleType().equals(
							"[package]")
							&& !osr2.getCode().equals("[generic]")
							&& osr2.getOrderItem() != null;
					if (itemInAnotherPackage) {
						if (log.isDebugEnabled())
							log.debug("Item in non-generic package: "
									+ osr2.getOrderItem().getCode());
						Integer itemID = new Integer(osr2.getOrderItem()
								.getOrderItemID());
						if (itemQuantitiesInOtherPackages.get(itemID) == null) {
							itemQuantitiesInOtherPackages.put(itemID,
									new Integer(osr2.getCeiling().intValue()));
						} else {
							Integer qty = (Integer) itemQuantitiesInOtherPackages
									.get(itemID);
							qty = new Integer(qty.intValue()
									+ osr2.getCeiling().intValue());
							itemQuantitiesInOtherPackages.put(itemID, qty);
						}
					}
				}
			}

			// Now loop through all items in the order and add up the weights of
			// those that are not accounted for in another package, but also
			// match the current drop shipper.
			Iterator j = getUser().getOrder().firstDelivery()
					.getOrderItemCollection().iterator();
			while (j.hasNext()) {
				OrderItem oi = (OrderItem) j.next();
				int qtyInGenericPackage = oi.getQuantity();
				Integer qtyInOtherPackages = (Integer) itemQuantitiesInOtherPackages
						.get(new Integer(oi.getOrderItemID()));
				if (qtyInOtherPackages != null) {
					qtyInGenericPackage = oi.getQuantity()
							- qtyInOtherPackages.intValue();
				}
				boolean itemMatchesDropShipper = currentDropShipperID != null
						&& oi.getManufacturer() != null
						&& oi.getManufacturerID().intValue() == currentDropShipperID
								.intValue();
				boolean itemNotDropShipped = currentDropShipperID == null
						&& (oi.getManufacturer() == null || !dropShipperIDs
								.contains(new Integer(oi.getManufacturer()
										.getManufacturerID())));
				boolean addTheWeight = qtyInGenericPackage > 0
						&& (itemMatchesDropShipper || itemNotDropShipped);
				if (log.isDebugEnabled())
					log.debug("Generic item booleans: itemMatchesDropShipper: "
							+ itemMatchesDropShipper + " itemNotDropShipped: "
							+ itemNotDropShipped + " addTheWeight: "
							+ addTheWeight + " qtyInGenericPackage: "
							+ qtyInGenericPackage);
				if (addTheWeight) {
					// Add only that portion of the order item's weight that's
					// in the generic package.
					double ratioOfGenPackage = new Double(qtyInGenericPackage)
							.doubleValue()
							/ new Double(oi.getQuantity()).doubleValue();
					if (log.isDebugEnabled())
						log.debug("qtyInGenericPackage: " + qtyInGenericPackage
								+ "; oi.getQuantity(): " + oi.getQuantity()
								+ "; ratioOfGenPackage: " + ratioOfGenPackage);
					double weightInGenPackage = ratioOfGenPackage
							* oi.getWeight().doubleValue();
					if (log.isDebugEnabled())
						log.debug("Adding package weight: " + ratioOfGenPackage
								+ " * " + oi.getWeight() + " = "
								+ weightInGenPackage);
					weight = weight + weightInGenPackage;
					foundAnItem = true;
				}
			}

			// Return null if this package has no items
			if (!foundAnItem) {
				return null;
			}
		}

		return new Double(weight);

	}

	public Collection mergeRates(Collection mergedRates, Collection rates) {
		if (mergedRates.isEmpty()) {
			return rates;
		}
		String currencyCode = (String) getSettings().getValue("currencyCode");
		Collection ratesToRemove = new ArrayList();
		Iterator i = mergedRates.iterator();
		while (i.hasNext()) {
			Map mergedRate = (Map) i.next();
			Iterator j = rates.iterator();
			boolean rateFound = false;
			while (j.hasNext()) {
				Map rate = (Map) j.next();
				if (rate.get("code").equals(mergedRate.get("code"))) {
					rateFound = true;
					Double priceDouble = (Double) rate.get("priceDouble");
					Double mergedPriceDouble = (Double) mergedRate
							.get("priceDouble");
					if (mergedPriceDouble == null) {
						mergedPriceDouble = new Double(0);
					}
					if (log.isInfoEnabled())
						log.info("Merging rate: " + rate.get("code"));
					if (log.isInfoEnabled())
						log.info("Existing merged rate: " + mergedPriceDouble);
					if (log.isInfoEnabled())
						log.info("New rate: " + priceDouble);
					double newPrice = priceDouble.doubleValue()
							+ mergedPriceDouble.doubleValue();
					if (log.isInfoEnabled())
						log
								.info("New merged rate before rounding: "
										+ newPrice);
					newPrice = utils().roundForMoney(newPrice).doubleValue();
					mergedRate.put("priceDouble", new Double(newPrice));
					mergedRate.put("originalPriceDouble", new Double(newPrice));
					mergedRate.put("price", formatPrice(newPrice, currencyCode,
							getUser().getLocale()));
					mergedRate.put("originalPrice", formatPrice(newPrice,
							currencyCode, getUser().getLocale()));
					if (log.isInfoEnabled())
						log.info("New merged rate: " + newPrice);
					continue;
				}
			}
			if (!rateFound) {
				if (log.isInfoEnabled())
					log.info("Removing rate: " + mergedRate);
				ratesToRemove.add(mergedRate);
			}
		}
		mergedRates.removeAll(ratesToRemove);
		return mergedRates;
	}

	public String doEstimatedDeliveryDateWithShipDateToday(Integer deliveryDays) {
		Calendar shipDate = Calendar.getInstance();

		// If it's past the cutoff time, we have to ship it tomorrow, not today
		String shippingCutoffHour = getSettings().getValue("shippingCutoffHour");
		if (shippingCutoffHour != null) {
			String now = formatDateTime(new java.util.Date()).substring(0, 20);
			String cutoff = now.substring(0, 10) + " " + shippingCutoffHour;
			boolean isPastCutoff = now.compareTo(cutoff) >= 0;
			if (isPastCutoff) {
				shipDate.add(Calendar.DATE, 1);
				log.info("Adding one day as it is past the cutoff time, delivery date is now: "
						+ formatDeliveryDate(shipDate));
			}
		}
		return doEstimatedDeliveryDate(shipDate, deliveryDays);
	}

	public String doEstimatedDeliveryDate(Calendar shipDate, Integer deliveryDays) {

		// Don't do anything if there's no delivery day provided
		if (deliveryDays == null || deliveryDays.intValue() < 1) {
			return null;
		}

		log.info("Initial ship date: " + formatDeliveryDate(shipDate));

		// If it's starting on a weekend or holiday, skip those
		shipDate = pushPastWeekendsAndHolidays(shipDate);

		// Add one day at a time and push past weekends/holidays each time
		for (int i = 0; i < deliveryDays; i++) {
			shipDate.add(Calendar.DATE, 1);
			log.info("Adding day number " + (i + 1) + " of " + deliveryDays + ", delivery date is now: "
					+ formatDeliveryDate(shipDate));
			shipDate = pushPastWeekendsAndHolidays(shipDate);
		}

		// Now format the date
		return formatDeliveryDate(shipDate);
	}

	protected String formatDeliveryDate(Calendar shipDate) {
		String monthString = Integer.toString(shipDate.get(Calendar.MONTH) + 1);
		if (monthString.length() == 1) {
			monthString = "0" + monthString;
		}
		String dayString = Integer
				.toString(shipDate.get(Calendar.DAY_OF_MONTH));
		if (dayString.length() == 1) {
			dayString = "0" + dayString;
		}
		String shipDateString = shipDate.get(Calendar.YEAR) + "-" + monthString
				+ "-" + dayString;
		return shipDateString;
	}


	protected Calendar pushPastWeekendsAndHolidays(Calendar shipDate) {
		// Push the delivery date to the next weekday and non-holiday
		if (shipDate.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) {
			shipDate.add(Calendar.DATE, 2);
			log.info("Adding two days for Saturday, delivery date is now: " + formatDeliveryDate(shipDate));
		} else if (shipDate.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
			shipDate.add(Calendar.DATE, 1);
			log.info("Adding one day for Sunday, delivery date is now: " + formatDeliveryDate(shipDate));
		}
		while (isBlackedOutHoliday(shipDate)) {
			shipDate.add(Calendar.DATE, 1);
			log.info("Adding one day for a Holiday, delivery date is now: " + formatDeliveryDate(shipDate));
		}
		return shipDate;
	}

	protected boolean isBlackedOutHoliday(Calendar c) {
		if (c != null) {
			int dayOfMonth = c.get(Calendar.DAY_OF_MONTH);
			int month = c.get(Calendar.MONTH);
			if (month == Calendar.JANUARY && dayOfMonth == 1) { // New Years
				return true;
			}
			if (month == Calendar.MAY) { // Memorial Day
				Calendar memorialDay = (Calendar) c.clone();
				memorialDay.set(Calendar.DAY_OF_WEEK, 2);
				memorialDay.set(Calendar.DAY_OF_WEEK_IN_MONTH, -1);
				if (dayOfMonth == memorialDay.get(Calendar.DAY_OF_MONTH)) {
					return true;
				}
			}
			if (month == Calendar.JULY && dayOfMonth == 4) { // 4th of July
				return true;
			}
			if (month == Calendar.SEPTEMBER) { // Labor Day
				Calendar laborDay = (Calendar) c.clone();
				laborDay.set(Calendar.DAY_OF_WEEK, 2);
				laborDay.set(Calendar.DAY_OF_WEEK_IN_MONTH, 1);
				if (dayOfMonth == laborDay.get(Calendar.DAY_OF_MONTH)) {
					return true;
				}
			}
			if (month == Calendar.NOVEMBER) { // Thanksgiving
				Calendar thanksgiving = (Calendar) c.clone();
				thanksgiving.set(Calendar.DAY_OF_WEEK, 5);
				thanksgiving.set(Calendar.DAY_OF_WEEK_IN_MONTH, 4);
				if (dayOfMonth == thanksgiving.get(Calendar.DAY_OF_MONTH)) {
					return true;
				}
			}
			if (month == Calendar.DECEMBER && dayOfMonth == 25) { // Christmas
				return true;
			}
		}
		return false;
	}

	public boolean orderHasGiftCertificatesOnly(Order order) {
		String giftCertificateAmountAttributeCode = (String) getSettings()
				.getValue("giftCertificateAmountAttributeCode");
		if (giftCertificateAmountAttributeCode == null || giftCertificateAmountAttributeCode.length() == 0) {
			giftCertificateAmountAttributeCode = "GiftCertificateAmount";
		}
		if (order == null
				|| order.firstDelivery() == null
				|| order.firstDelivery().getOrderItemCollection() == null) {
			return false;
		}
		Iterator i = order.firstDelivery().getOrderItemCollection().iterator();
		while (i.hasNext()) {
			OrderItem oi = (OrderItem) i.next();
			if (oi.getOrderItemAttributes() == null) {
				return false;
			}
			boolean foundGiftCertAttribute = false;
			Iterator j = oi.getOrderItemAttributes().iterator();
			while (j.hasNext()) {
				OrderItemAttributeBean oia = (OrderItemAttributeBean) j.next();
				if (oia.getAttributeCode().equals(
						giftCertificateAmountAttributeCode)) {
					foundGiftCertAttribute = true;
				}
			}
			if (!foundGiftCertAttribute) {
				return false;
			}
		}
		return true;
	}
}
